// tagliner by mcree - my first linux C code 8)

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

// maximal line length+1
#define MAX_LENGTH 1024
// random seed filename base (current uid is appended to avoid permission conflicts)
#define SEED_FILEBASE "/tmp/tagliner.%i.seed"

void fatal(char *errorcode)
{
    printf(errorcode);
    exit(1);
};

void tagliner(char *ifile, char *ofile, int split)
{
    FILE *tagfile;
    FILE *outfile = stdout;
    char *s = alloca(MAX_LENGTH);
    char *ss;
    int le = 0;
    unsigned long int taglines = 0, actual = 0;

    FILE *seedfile;
    char *seed_filename = alloca(40);
    struct drand48_data *BUFFER = alloca(sizeof(struct drand48_data));

    memset(BUFFER, '\0', sizeof(struct drand48_data));	// zero random buffer


    snprintf(seed_filename, 40, SEED_FILEBASE, getuid());
    seedfile = fopen(seed_filename, "r");
    if (seedfile == NULL) {
	if (errno == ENOENT) {
	    srand48_r(time(0), BUFFER);	// randomize
	    seedfile = fopen(seed_filename, "w+");	// ignore other errors... we'll just randomize
	    if (seedfile > 0) {
		fwrite(BUFFER, sizeof(struct drand48_data), 1, seedfile);
		fclose(seedfile);
		chmod(seed_filename, S_IWUSR | S_IRUSR);
	    };
	} else {
	    perror("Seed file open error");
	};
    } else {
	if (fread(BUFFER, sizeof(struct drand48_data), 1, seedfile) == 0)
	    srand48_r(time(0), BUFFER);	// ignore errors
	fclose(seedfile);
    };

    // open files
    if (!(tagfile = fopen(ifile, "r")))
	fatal("Input file open error\n");

    // count input lines        
    do {
	fgets(s, MAX_LENGTH - 1, tagfile);
	taglines++;
    }
    while (!feof(tagfile));

    // choose and read random line      
//    actual = (lrand48() % taglines);
    lrand48_r(BUFFER, &actual);
    actual %= taglines;

    seedfile = fopen(seed_filename, "w+");	// ignore other errors... we'll just randomize
    if (seedfile > 0) {
	fwrite(BUFFER, sizeof(struct drand48_data), 1, seedfile);
	fclose(seedfile);
    } else {
	perror("Seed file open error");
    };


    fseek(tagfile, 0, SEEK_SET);
    taglines = 0;

    do {
	fgets(s, MAX_LENGTH - 1, tagfile);
	taglines++;
    }
    while ((!feof(tagfile)) && (actual != taglines));

    // if second parameter is given     
    if (ofile != NULL) {
	if (strcmp(ofile, "stdout"))
	    if (!(outfile = fopen(ofile, "a")))
		fatal("Output file open error\n");
    };

    // echo random line to stdout
    if ((split == 0) || (split > MAX_LENGTH - 1)) {
	fputs(s, outfile);
    } else {

	s[strlen(s) - 1] = 0;

	ss = strtok(s, " ");

	do {

	    if (le + strlen(ss) + 1 > split) {
		le = strlen(ss) + 1;
		fprintf(outfile, "\n %s", ss);
	    } else {
		le += strlen(ss) + 1;
		fprintf(outfile, " %s", ss);
	    };

	} while ((ss = strtok(NULL, " ")) != NULL);

	fprintf(outfile, "\n");

    };


    fclose(tagfile);

    // close output file if difers from stdout
    if (outfile != stdout)
	fclose(outfile);
};

int main(int ARGC, char *ARGV[])
{
    int splitvalue = 0;

    if (ARGC < 2)
	fatal("tagliner 0.3 - tagline randomizer by McRee (1999-2000)\n\n"
	      "Invalid argument(s)!\n\n"
	      "usage: tagliner infile [outfile [split]]\n\n"
	      "         infile  - input file containing one tagline per row\n"
	      "         outfile - the tagline will be appended to this file\n"
	      "                   if nothing is specified 'stdout' is used instead\n"
	      "         split   - optionally splits output lines when longer\n"
	      "                   than the value you specified. Invalid values\n"
	      "                   are ignored without warning\n");
    else {
	if (ARGV[3] != NULL)
	    splitvalue = strtol(ARGV[3], NULL, 10);
	tagliner(ARGV[1], ARGV[2], abs(splitvalue));	// if there is only one parameter ARGV[2] and ARGV[3] are still NULL
    };

    return EXIT_SUCCESS;
};
